#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from onf_website.shared.leisure.interfaces import IWfLeisure, LEISURE_CONTENT_TYPE, LEISURE_CONTENT_NAME, ILeisure, \
    ILeisureInfo, LEISURE_INFO_ANNOTATIONS_KEY
from pyams_content.component.theme.interfaces import IThemesTarget
from pyams_content.features.preview.interfaces import IPreviewTarget
from pyams_content.features.review.interfaces import IReviewTarget
from zope.annotation.interfaces import IAnnotations

# import packages
from persistent import Persistent
from pyams_content.shared.common import WfSharedContent, register_content_type, SharedContent
from pyams_utils.adapter import adapter_config
from pyams_utils.registry import get_global_registry
from zope.container.contained import Contained
from zope.interface import implementer
from zope.lifecycleevent import ObjectCreatedEvent
from zope.location import locate
from zope.schema.fieldproperty import FieldProperty


@implementer(IWfLeisure, IThemesTarget, IPreviewTarget, IReviewTarget)
class WfLeisure(WfSharedContent):
    """Leisure class"""

    content_type = LEISURE_CONTENT_TYPE
    content_name = LEISURE_CONTENT_NAME

register_content_type(WfLeisure)


@implementer(ILeisureInfo)
class LeisureInfo(Persistent, Contained):
    """Leisure persistent informations"""

    full = FieldProperty(ILeisureInfo['full'])
    favorite = FieldProperty(ILeisureInfo['favorite'])
    priority = FieldProperty(ILeisureInfo['priority'])


@adapter_config(context=IWfLeisure, provides=ILeisureInfo)
def leisure_info_factory(context):
    """Leisure info factory"""
    annotations = IAnnotations(context)
    info = annotations.get(LEISURE_INFO_ANNOTATIONS_KEY)
    if info is None:
        info = annotations[LEISURE_INFO_ANNOTATIONS_KEY] = LeisureInfo()
        registry = get_global_registry()
        registry.notify(ObjectCreatedEvent(info))
        locate(info, context, '++info++')
    return info


@implementer(ILeisure)
class Leisure(SharedContent):
    """Workflow managed leisure"""

    content_class = WfLeisure
